"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.scheduler = exports.Scheduler = exports.debounceTime = void 0;
exports.resetScheduler = resetScheduler;
const event_1 = require("./event");
const transmitter_1 = require("../transmitter");
const client_1 = require("../client");
const utils_1 = require("../utils");
const INITIAL_DEBOUNCE_MILLISECONDS = 100;
const BETWEEN_TRANSMISSIONS_DEBOUNCE_MILLISECONDS = 10000;
/** @internal */
exports.debounceTime = (0, utils_1.stubbable)((lastTransmission) => {
    if (lastTransmission === undefined) {
        return INITIAL_DEBOUNCE_MILLISECONDS;
    }
    const elapsed = Date.now() - lastTransmission;
    return Math.max(INITIAL_DEBOUNCE_MILLISECONDS, BETWEEN_TRANSMISSIONS_DEBOUNCE_MILLISECONDS - elapsed);
});
class PendingPromiseSet extends Set {
    add(promise) {
        super.add(promise);
        promise.finally(() => this.delete(promise));
        return this;
    }
    async allSettled() {
        await Promise.allSettled(this);
    }
}
/** @internal */
class Scheduler {
    constructor() {
        this.pendingTransmissions = new PendingPromiseSet();
        this.events = [];
        this.isShuttingDown = false;
    }
    async shutdown() {
        this.isShuttingDown = true;
        this.runWaker();
        await this.allSettled();
    }
    allSettled() {
        return this.pendingTransmissions.allSettled();
    }
    schedule(event) {
        if (client_1.Client.client === undefined || !client_1.Client.client.isActive) {
            client_1.Client.internalLogger.debug(`Cannot schedule ${event_1.Event.describe([event])}: AppSignal is not active`);
            return;
        }
        if (this.isShuttingDown) {
            client_1.Client.internalLogger.debug(`Cannot schedule ${event_1.Event.describe([event])}: AppSignal is stopped`);
            return;
        }
        client_1.Client.internalLogger.trace(`Scheduling ${event_1.Event.describe([event])}`);
        this.addEvent(event);
        this.scheduleWaker();
    }
    addEvent(event) {
        // Remove redundant events, keeping the newly added one, which
        // should be the one with the most recent timestamp.
        this.events = this.events.filter(existingEvent => {
            const isRedundant = existingEvent.isRedundant(event);
            if (isRedundant) {
                client_1.Client.internalLogger.debug(`Replacing previously scheduled ${event_1.Event.describe([existingEvent])}`);
            }
            return !isRedundant;
        });
        this.events.push(event);
    }
    scheduleWaker() {
        if (this.waker === undefined) {
            this.waker = setTimeout(() => this.runWaker(), (0, exports.debounceTime)(this.lastTransmission));
            // Ensure the debounce period is not awaited when the Node.js process is
            // shutting down -- users must call Appsignal.stop() to gracefully await the
            // transmission of scheduled events.
            this.waker.unref();
        }
    }
    runWaker() {
        this.lastTransmission = Date.now();
        if (this.waker !== undefined) {
            clearTimeout(this.waker);
            this.waker = undefined;
        }
        const events = this.events;
        event_1.Event.deduplicateCron(events);
        this.events = [];
        if (events.length !== 0) {
            // The events array may be empty when this function is called on shutdown.
            this.transmit(events);
        }
    }
    transmit(events) {
        const description = event_1.Event.describe(events);
        const promise = new transmitter_1.Transmitter(`${client_1.Client.config.data.loggingEndpoint}/check_ins/json`, (0, utils_1.ndjsonStringify)(events)).transmit();
        const handledPromise = promise
            .then(({ status }) => {
            if (status >= 200 && status <= 299) {
                client_1.Client.internalLogger.trace(`Transmitted ${description}`);
            }
            else {
                client_1.Client.internalLogger.error(`Failed to transmit ${description}: status code was ${status}`);
            }
        })
            .catch(({ error }) => {
            client_1.Client.internalLogger.error(`Failed to transmit ${description}: ${error.message}`);
            return Promise.resolve();
        });
        this.pendingTransmissions.add(handledPromise);
    }
}
exports.Scheduler = Scheduler;
/** @internal */
exports.scheduler = new Scheduler();
/** @internal */
async function resetScheduler() {
    await exports.scheduler.shutdown();
    exports.scheduler = new Scheduler();
}
