import { AppsignalOptions } from "./config/options";
import { Extension } from "./extension";
import { Configuration } from "./config";
import { Metrics } from "./metrics";
import { InternalLogger } from "./internal_logger";
import { Logger, LoggerFormat, LoggerLevel } from "./logger";
import { AmqplibInstrumentation } from "@opentelemetry/instrumentation-amqplib";
import { BullMQInstrumentation } from "@appsignal/opentelemetry-instrumentation-bullmq";
import { ExpressInstrumentation } from "@opentelemetry/instrumentation-express";
import { FastifyInstrumentation } from "@opentelemetry/instrumentation-fastify";
import { GraphQLInstrumentation } from "@opentelemetry/instrumentation-graphql";
import { HttpInstrumentation } from "@opentelemetry/instrumentation-http";
import { IORedisInstrumentation } from "@opentelemetry/instrumentation-ioredis";
import { KnexInstrumentation } from "@opentelemetry/instrumentation-knex";
import { KoaInstrumentation } from "@opentelemetry/instrumentation-koa";
import { MongoDBInstrumentation } from "@opentelemetry/instrumentation-mongodb";
import { MongooseInstrumentation } from "@opentelemetry/instrumentation-mongoose";
import { MySQL2Instrumentation } from "@opentelemetry/instrumentation-mysql2";
import { MySQLInstrumentation } from "@opentelemetry/instrumentation-mysql";
import { NodeSDKConfiguration } from "@opentelemetry/sdk-node";
import { NestInstrumentation } from "@opentelemetry/instrumentation-nestjs-core";
import { PgInstrumentation } from "@opentelemetry/instrumentation-pg";
import { PrismaInstrumentation } from "@prisma/instrumentation";
import { RedisInstrumentation as Redis4Instrumentation } from "@opentelemetry/instrumentation-redis-4";
import { RedisInstrumentation } from "@opentelemetry/instrumentation-redis";
import { RestifyInstrumentation } from "@opentelemetry/instrumentation-restify";
import { UndiciInstrumentation } from "@opentelemetry/instrumentation-undici";
declare const DefaultInstrumentations: {
    "@appsignal/opentelemetry-instrumentation-bullmq": typeof BullMQInstrumentation;
    "@opentelemetry/instrumentation-amqplib": typeof AmqplibInstrumentation;
    "@opentelemetry/instrumentation-express": typeof ExpressInstrumentation;
    "@opentelemetry/instrumentation-fastify": typeof FastifyInstrumentation;
    "@opentelemetry/instrumentation-graphql": typeof GraphQLInstrumentation;
    "@opentelemetry/instrumentation-http": typeof HttpInstrumentation;
    "@opentelemetry/instrumentation-ioredis": typeof IORedisInstrumentation;
    "@opentelemetry/instrumentation-knex": typeof KnexInstrumentation;
    "@opentelemetry/instrumentation-koa": typeof KoaInstrumentation;
    "@opentelemetry/instrumentation-mongodb": typeof MongoDBInstrumentation;
    "@opentelemetry/instrumentation-mongoose": typeof MongooseInstrumentation;
    "@opentelemetry/instrumentation-mysql2": typeof MySQL2Instrumentation;
    "@opentelemetry/instrumentation-mysql": typeof MySQLInstrumentation;
    "@opentelemetry/instrumentation-nestjs-core": typeof NestInstrumentation;
    "@opentelemetry/instrumentation-pg": typeof PgInstrumentation;
    "@opentelemetry/instrumentation-redis": typeof RedisInstrumentation;
    "@opentelemetry/instrumentation-redis-4": typeof Redis4Instrumentation;
    "@opentelemetry/instrumentation-restify": typeof RestifyInstrumentation;
    "@opentelemetry/instrumentation-undici": typeof UndiciInstrumentation;
    "@prisma/instrumentation": typeof PrismaInstrumentation;
};
export type DefaultInstrumentationName = keyof typeof DefaultInstrumentations;
type NodeSDKInstrumentationsOption = NodeSDKConfiguration["instrumentations"];
export type Options = AppsignalOptions & {
    additionalInstrumentations: NodeSDKInstrumentationsOption;
};
/**
 * AppSignal for Node.js's main class.
 *
 * Provides methods to control the AppSignal instrumentation and the system
 * agent.
 *
 * @class
 */
export declare class Client {
    #private;
    readonly VERSION: any;
    config: Configuration;
    readonly internalLogger: InternalLogger;
    extension: Extension;
    /**
     * Global accessors for the AppSignal client
     */
    static get client(): Client;
    /**
     * Global accessors for the AppSignal Config
     */
    static get config(): Configuration;
    /**
     * Global accessors for the AppSignal integration Logger
     */
    static get internalLogger(): InternalLogger;
    /**
     * Global accessors for the AppSignal Logger API
     */
    static logger(group: string, level?: LoggerLevel, format?: LoggerFormat): Logger;
    /**
     * Starts AppSignal with the given configuration. If no configuration is set
     * yet it will try to automatically load the configuration using the
     * environment loaded from environment variables.
     */
    constructor(options?: Partial<Options>);
    get isActive(): boolean;
    /**
     * Stops the AppSignal agent.
     *
     * Call this before the end of your program to make sure the
     * agent is stopped as well.
     */
    stop(calledBy?: string): Promise<void>;
    /**
     * Internal private function used by the demo CLI.
     *
     * https://docs.appsignal.com/nodejs/3.x/command-line/demo.html
     *
     * @private
     */
    demo(): void;
    /**
     * Returns the current `Metrics` object.
     *
     * To track application-wide metrics, you can send custom metrics to AppSignal.
     * These metrics enable you to track anything in your application, from newly
     * registered users to database disk usage. These are not replacements for custom
     * instrumentation, but provide an additional way to make certain data in your
     * code more accessible and measurable over time.
     *
     * With different types of metrics (gauges, counters and measurements)
     * you can track any kind of data from your apps and tag them with metadata
     * to easily spot the differences between contexts.
     */
    metrics(): Metrics;
    logger(group: string, level?: LoggerLevel, format?: LoggerFormat): Logger;
    /**
     * Initialises all the available probes to attach automatically at runtime.
     */
    private initCoreProbes;
    private defaultInstrumentationsConfig;
    private defaultInstrumentations;
    opentelemetryInstrumentations(): NodeSDKInstrumentationsOption;
    /**
     * Initialises OpenTelemetry instrumentation
     */
    private initOpenTelemetry;
    /**
     * Sets up the AppSignal logger with the output based on the `log` config option. If
     * the log file is not accessible, stdout will be the output.
     */
    private setupInternalLogger;
    /**
     * Sets up the OpenTelemetry diag logger based on our integration logger level.
     * If our integration logger level is "silly" ("trace"), the OpenTelemetry diag debug messages
     * are logged.
     */
    private setUpOpenTelemetryLogger;
    /**
     * Stores the client in global object after initializing
     */
    private storeInGlobal;
    get tracer(): any;
}
export {};
