"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.setParams = setParams;
exports.setSessionData = setSessionData;
exports.setCustomData = setCustomData;
exports.setTag = setTag;
exports.setHeader = setHeader;
exports.setName = setName;
exports.setCategory = setCategory;
exports.setBody = setBody;
exports.setSqlBody = setSqlBody;
exports.setNamespace = setNamespace;
exports.setRootName = setRootName;
exports.setError = setError;
exports.sendError = sendError;
exports.instrumentationsLoaded = instrumentationsLoaded;
const api_1 = require("@opentelemetry/api");
const client_1 = require("./client");
function setAttribute(attribute, value, span) {
    const activeSpan = span !== null && span !== void 0 ? span : api_1.trace.getActiveSpan();
    if (activeSpan) {
        activeSpan.setAttribute(attribute, value);
    }
    else {
        const splitAttributes = attribute.split(".");
        const attributeSuffix = splitAttributes[splitAttributes.length - 1];
        client_1.Client.internalLogger.warn(`There is no active span, cannot set \`${attributeSuffix}\``);
    }
}
function circularReplacer() {
    const seenValue = [];
    const seenKey = [];
    return (key, value) => {
        if (typeof value === "object" && value !== null) {
            const i = seenValue.indexOf(value);
            if (i !== -1) {
                return `[cyclic value: ${seenKey[i] || "root object"}]`;
            }
            else {
                seenValue.push(value);
                seenKey.push(key);
            }
        }
        return value;
    };
}
function setSerialisedAttribute(attribute, value, span) {
    const serialisedValue = JSON.stringify(value, circularReplacer());
    if (serialisedValue) {
        setAttribute(attribute, serialisedValue, span);
    }
}
function setPrefixedAttribute(prefix, suffix, value, span) {
    if (suffix) {
        setAttribute(`${prefix}.${suffix}`, value, span);
    }
}
function setParams(params, span) {
    setSerialisedAttribute("appsignal.request.parameters", params, span);
}
function setSessionData(sessionData, span) {
    setSerialisedAttribute("appsignal.request.session_data", sessionData, span);
}
function setCustomData(customData, span) {
    setSerialisedAttribute("appsignal.custom_data", customData, span);
}
function setTag(tag, value, span) {
    setPrefixedAttribute("appsignal.tag", tag, value, span);
}
function setHeader(header, value, span) {
    setPrefixedAttribute("appsignal.request.headers", header, value, span);
}
function setName(name, span) {
    setAttribute("appsignal.name", name, span);
}
function setCategory(category, span) {
    setAttribute("appsignal.category", category, span);
}
function setBody(body, span) {
    setAttribute("appsignal.body", body, span);
}
function setSqlBody(body, span) {
    setAttribute("appsignal.sql_body", body, span);
}
function setNamespace(namespace, span) {
    setAttribute("appsignal.namespace", namespace, span);
}
function setRootName(name, span) {
    setAttribute("appsignal.root_name", name, span);
}
function setError(error, span) {
    if (error && error.name && error.message) {
        const activeSpan = span !== null && span !== void 0 ? span : api_1.trace.getActiveSpan();
        if (activeSpan) {
            activeSpan.recordException(error);
            activeSpan.setStatus({
                code: api_1.SpanStatusCode.ERROR,
                message: error.message
            });
        }
        else {
            client_1.Client.internalLogger.debug(`There is no active span, cannot set \`${error.name}\``);
        }
    }
    else {
        client_1.Client.internalLogger.debug("Cannot set error, it is not an `Error`-like object");
    }
}
function sendError(error, fn = () => { }) {
    if (error && error.name && error.message) {
        api_1.trace
            .getTracer("Appsignal.sendError")
            .startActiveSpan(error.name, { root: true }, span => {
            setError(error);
            fn();
            span.end();
        });
    }
    else {
        client_1.Client.internalLogger.debug("Cannot send error, it is not an `Error`-like object");
    }
}
/**
 * @deprecated This function is no longer required for manual instrumentation.
 */
function instrumentationsLoaded() {
    client_1.Client.internalLogger.warn("instrumentationsLoaded() is deprecated, please remove it from your code as it'll be deleted in the next major release.");
    return Promise.resolve();
}
