"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAgentTimestamps = getAgentTimestamps;
exports.isWritable = isWritable;
exports.hrTime = hrTime;
exports.processGetuid = processGetuid;
exports.ndjsonStringify = ndjsonStringify;
exports.ndjsonParse = ndjsonParse;
exports.stubbable = stubbable;
const tslib_1 = require("tslib");
const fs_1 = tslib_1.__importDefault(require("fs"));
const perf_hooks_1 = tslib_1.__importDefault(require("perf_hooks"));
const NANOSECOND_DIGITS = 9;
const SECOND_TO_NANOSECONDS = Math.pow(10, NANOSECOND_DIGITS);
/**
 * Given a valid POSIX `timestamp` in milliseconds since the UNIX epoch,
 * return an object containing a representation if that timestamps in
 * seconds and nanoseconds.
 *
 * @function
 * @internal
 */
function getAgentTimestamps(timestamp) {
    const sec = Math.round(timestamp / 1000);
    return {
        sec: sec, // seconds
        nsec: timestamp * 1e6 - sec * 1e9 // nanoseconds
    };
}
/**
 * Checks if the given path is writable by the process.
 * @internal
 */
function isWritable(path) {
    try {
        fs_1.default.accessSync(path, fs_1.default.constants.W_OK);
        return true;
    }
    catch (e) {
        return false;
    }
}
/**
 * Returns a high-resolution time tuple containing the current time in seconds and nanoseconds.
 *
 * @function
 * @internal
 */
function hrTime(performance = perf_hooks_1.default.performance) {
    const origin = numberToHrtime(performance.timeOrigin);
    const now = numberToHrtime(performance.now());
    return { sec: origin[0] + now[0], nsec: origin[1] + now[1] };
}
function numberToHrtime(epochMillis) {
    const epochSeconds = epochMillis / 1000;
    const seconds = Math.trunc(epochSeconds);
    const nanoseconds = Number((epochSeconds - seconds).toFixed(NANOSECOND_DIGITS)) *
        SECOND_TO_NANOSECONDS;
    return [seconds, nanoseconds];
}
/** @internal */
function processGetuid() {
    var _a;
    return ((_a = process.getuid) !== null && _a !== void 0 ? _a : (() => -1))();
}
/** @internal */
function ndjsonStringify(elements) {
    return elements.map(element => JSON.stringify(element)).join("\n");
}
/** @internal */
function ndjsonParse(data) {
    return data.split("\n").map(line => JSON.parse(line));
}
/** @internal */
function stubbable(original) {
    let current = original;
    const stub = {
        [original.name]: function (...args) {
            return current(...args);
        }
    }[original.name];
    function set(fn) {
        current = fn;
    }
    function reset() {
        current = original;
    }
    stub.set = set;
    stub.reset = reset;
    return stub;
}
